dnl ********************************************************
dnl --Title	: LIBRARY VHDL OF COMPONENTS USE IN THIS APPLICATION
dnl --AUTHOR 	: Pierre NIANG 
dnl --COMPANY	: ESIEE
dnl --DEPARTMENT: A2SI

define(`_Partielpackage',
type  VECTOR$1 is array (natural RANGE <>) of std_logic_vector($2-1 downto 0);
`ifelse($#,2,,`_Partielpackage(shift(shift($@)))')
')

define(`_Package',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;

--*****************   DEFINITION of PACKAGE ******************
--ce module permet de definir les dimmensions du package of application : $1
package definitions is
`_Partielpackage(shift($@))'
end definitions; 
)

define(`_Title',-------------------------------------------------------------------------------
-- Title       : AUTOMATIC GENERATION OF VHDL CODE FOR TOP ALGO $1
-- Project     : Automated generation of VHDL with module Gen_vhdl
-------------------------------------------------------------------------------
-- File        : $1_toplevel.vhdl
-- Author      : Pierre NIANG
-- Company     : ESIEE					
-- Department  : A2SI
)



define(`_Gen_lib',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;
)

define(`_Gen_entity_uci',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;

--************************ DEFINITION DE UC INFINI (entity et architecture)  ******************************
entity uc_infini is
  port (        rst  : in std_logic;
                en   : out std_logic;
		rfd  : out std_logic;
		afu  : out std_logic;
		rsu  : in std_logic;
		asd  : in std_logic;
		rfu  : in std_logic;
		afd  : in std_logic);
end uc_infini;

architecture OPERATEUR of uc_infini is

Begin
      
      rfd <= rsu;
      afu <= asd;
      en  <= afd OR rst;
      
end OPERATEUR;
)

define(`_Gen_entity_uc',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;

--***************************** DEFINITION DE UC (entity et architecture)  ****************************
entity uc is
generic (nb_bit_compteur :integer:=6 );
port (rsu,afd,rfu,asd, clk,reset    :in std_logic;
      asu,rfd,afu,rsd,en            :out std_logic;
      cpt                           :out std_logic_vector (nb_bit_compteur-1 downto 0));

end uc;
architecture ucff of uc is
signal compteur: std_logic_vector (nb_bit_compteur-1 downto 0):= (others => '0'); --:="000001";
signal tran    : std_logic:='0';
signal fin     : std_logic;
signal init    : std_logic;
signal enable  : std_logic;
begin

--description de unite de controle
en  <= enable;
fin <= compteur(nb_bit_compteur-1);
rfd <= rsu;
asu <= afd and fin;
cpt <=compteur;
enable  <= (rsu and afd) and ((not fin) or asd);
rsd <= rfu and fin;
afu <= reset or (rsu and (not fin)) or (asd or ((not fin) and (rsu and afd)));
init <= reset or ((afd and fin) and asd);

--description du compteur one hot encoding
process (init, clk, enable)
begin
 if (clk' event and clk = '1') and ( init = '1') then
	       compteur(nb_bit_compteur-1 downto 1) <= (others => '0') ;
	       compteur(0) <= '1';

 else if (clk' event and clk = '1') and (enable = '1') then
		tran <= compteur(nb_bit_compteur-1);
		compteur(nb_bit_compteur-1 downto 1) <= compteur (nb_bit_compteur-2 downto 0);
		compteur(0) <= tran;
	end if;
 end if;
end process;

end ucff;
)			      

define(`_Gen_entity_mseq',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--**************************  DEFINITION  Mseq OPERATION   ***** **************************
entity $1Mseq is
        generic(        insize:integer :=4;
	                outsize:integer :=12;
			nel:integer :=3;
			tel:integer :=8);

	port (
		rst : std_logic;
		inport : in VECTOR$1 (insize-1 downto 0) ;
		outport : out VECTOR$1 (outsize-1 downto 0) ;
		cpt : in std_logic_vector(nel-1 downto 0);
		en  : in std_logic;
		clk : in std_logic);
end $1Mseq;

architecture OPERATEUR of $1Mseq is


signal  sortie_tmp : VECTOR$1 (outsize-1 downto 0) ;
signal  sortie_tmp2 : VECTOR$1 (insize-1 downto 0) ;

begin

process(sortie_tmp, sortie_tmp2)
	begin
		for i in 0 to outsize - 2 loop
			 outport(i) <= sortie_tmp(i);
		end loop;
		outport(outsize-1 downto (outsize-insize)) <= sortie_tmp2;
	end process;



LAST: process(inport)
	begin
		sortie_tmp2 <= inport;
																						 end process;

IMPLO : process (clk, en)
	begin
          if (clk'event and clk = '1') then

                       if (en = '1') then
		          for i in 0 to nel - 1 loop
				if (cpt(i) = '1') then
					sortie_tmp(((insize*(i+1)) -1) downto (i*insize)) <= inport(insize-1 downto 0);
				else
					 sortie_tmp((insize*(i-1)) downto (i*insize)) <= sortie_tmp((insize*(i-1)) downto (i*insize));
				end if;
			  end loop;

			 end if;

	   end if;
        end process;


end OPERATEUR;
)

define(`_Gen_entity_xseq',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--**************************   DEFINITION Xseq OPERATION    ********************************

entity $1oneXseq is

        generic(       insize : integer := 8;
	                tel : integer := 8);
	port (       Entree : in  VECTOR$1 (insize-1 downto 0) ;
		     sortie : out std_logic_vector(tel-1 downto 0);
		     Commande : in std_logic_vector(insize-1 downto 0)
		     );
end $1oneXseq;

architecture OPERATEUR of $1oneXseq is


begin



  IMPLO : process (Entree,Commande)
          variable tmp : VECTOR$1 ( insize-1 downto 0) ;
	  variable tmp_ou : std_logic_vector(insize-1 downto 0);

  	begin
	for i in 0 to (insize - 1) loop
	                for j in 0 to (tel - 1) loop
			           tmp(i)(j) := Entree(i)(j) AND Commande(i);
			end loop;
	end loop;
	for i in 0 to (tel - 1) loop
	                 tmp_ou(0) := tmp(0)(i);
			 for j in 1 to (insize - 1) loop
				tmp_ou(j) := tmp_ou(j-1) OR tmp(j)(i);
			 end loop;
			 Sortie(i) <= tmp_ou(insize-1);
	end loop;

   end process;

end OPERATEUR;

--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;

entity $1Xseq is


generic(  insize:integer := 12;
          outsize:integer := 4;
	  nel:integer   :=3;
	  tel:integer   :=8);

port (          rst : in std_logic;
		inport: in VECTOR$1 (insize -1 downto 0) ;
		outport: out VECTOR$1 (outsize -1 downto 0) ;
		cpt : in std_logic_vector(nel-1 downto 0)
	);
end $1Xseq ;

architecture Xseq_arch of $1Xseq is

type   VECTOR_COM is array (natural RANGE <>) of std_logic_vector(insize-1 downto 0);

signal commtemp : std_logic_vector(insize-1 downto 0);
signal tempcom  : VECTOR_COM(0 to outsize-1);


        Component $1oneXseq
	        generic(    insize : integer := 8;
		                   tel : integer := 8);
		port (
				Entree : in  VECTOR$1 (insize-1 downto 0);
				sortie : out std_logic_vector(tel-1 downto 0);
				Commande : in std_logic_vector(insize-1 downto 0)
			);
	End component;

begin
        Xseq1 : for i in 0 to outsize -1 generate
	        realXseq: if (insize-outsize>0) generate
			littleXseq:$1oneXseq
				 generic map(insize,tel)
				 port map (inport, outport(i), tempcom(i));
		end generate realXseq;
		diffuse: if (insize-outsize=0) generate
				outport(i) <= inport(i);
		end generate diffuse;

	end generate Xseq1;
sigcom : process(cpt,tempcom,commtemp)
variable iter : integer :=((insize-outsize)/(nel-1));
begin
        if cpt'event then
	       for i in 0 to nel-1 loop
			if cpt(i)='1' then
				tempcom(0)<=('0',others=>'0');
				tempcom(0)((iter*i))<='1';
			end if;
		end loop;
	end if;
	for i in 1 to outsize-1 loop
			tempcom(i)<=tempcom(i-1)(insize-2 downto 0) & '0';
	end loop;
	end process sigcom;

end Xseq_arch;
		    
)

define(`_Gen_entity_iseq',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--***************************   DEFINITION Iseq OPERATION  *********************************

entity $1Iterate is
  generic (     size  : integer :=1;
                nel : integer :=3;
		tel : integer :=8
	);
  port (        rst : in std_logic;
		I_init : in VECTOR$1 (size-1 downto 0) ;
		inport  : in VECTOR$1 (size-1 downto 0) ;
		O_si  : out VECTOR$1 (size-1 downto 0) ;
		outport  : out VECTOR$1 (size-1 downto 0) ;
		cpt: in std_logic_vector(nel-1 downto 0);
		en  : in std_logic;
		clk : in std_logic
	);
end $1Iterate;

architecture OPERATEUR of $1Iterate is

  signal Retard  : VECTOR$1 (size -1  downto 0) ;

  begin
      I : process (clk)
          begin
	        if (clk'event and (clk = '1')) then
		        if (en = '1') then
			          Retard <= inport;
			end if;
		end if;
	end process I;
			
    process(cpt)
      begin
              if (cpt(0) = '0') then
	                      O_si <= Retard;
	      else
			      O_si <= I_init;
	      end if;
     end process;

     outport <= inport;
end OPERATEUR;
																			
)

define(`_Const_entity_core',`ifelse($#,3,,
	`$1(incr($2)) <= conv_std_logic_vector ($4,tel1);
	_Const_entity_core($1,incr($2),shift(shift(shift($@))))')
')


define(`_Gen_entity_const',
`ifelse($#,incr(incr(incr(incr(incr(incr($4)))))), 
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--***************************  DEFINITION CONSTANTE OPERATION  *******************************

entity $1$2 is
      	generic(          size1 : integer :=$4;
                         tel1 : integer :=$5
		);
	port (
		$3   : out VECTOR$6 (size1-1 downto 0)
		);
	end $1$2;

architecture OPERATEUR of $1$2 is
begin
	$3(0) <= conv_std_logic_vector ($7,tel1);
	_Const_entity_core($3,0,shift(shift(shift(shift(shift(shift($@)))))))
end OPERATEUR;, pierre--COMPONENT $1$2 NOT DEFINED; COMPILE IT SEPARATLY
		_Gen_entity_const_not_in_lib($@) )

')

define(`_Gen_entity_memory',
`ifelse($#,incr(incr(incr(incr(incr(incr(incr(incr(incr(incr(incr(incr($5)))))))))))),
--********************  DEFINITION OF MEMORY  **********************************
LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;

--**************************  DEFINITION OF MEMORY OPERATION *********************

entity $1$2 is 
	generic(	size1 : integer := $5;
			tel1 : integer := $6;
			size2 : integer := $10;
			tel2 : integer := $11
		);
	port(
			$3 : $4 VECTOR$7(size1 -1 downto 0);
			$8 : $9 VECTOR$12(size2 -1 downto 0);
			en : in std_logic;
			clk : in std_logic;
			rst : in std_logic
	    );
end $1$2;

architecture OPERATEUR of $1$2 is
signal tabvect : VECTOR$7(size2-1 downto 0) ;
begin

$8 <= tabvect ;
process(clk,rst,en)
  begin
    if rst = ''1'` then
	tabvect(0) <= conv_std_logic_vector ($13,tel1);
	_Const_entity_core(tabvect,0,shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))
    elsif '(clk'event and clk = '1') then 
    	if en = '1'` then
		tabvect <= $3;
	end if;
    end if;
end process;
end OPERATEUR;, --COMPONENT $1$2 NOT DEFINED; COMPILE IT SEPARATLY
		_Gen_entity_memory_not_in_lib($@))

')
	
define(`_Gen_entity_actu_partiel',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--****************************  DEFINITION ACTUATOR OPERATION  ****************************

entity $1$2 is
        generic(          size1 : integer :=$4;
	                  tel1 : integer :=$5
		);
	port (
			$3  : in VECTOR$6(size1-1 downto 0);
			en  : in std_logic;
			clk : in std_logic;
			rst : in std_logic
		);
	end $1$2;
	
architecture OPERATEUR of $1$2 is

type tableau is array (9 downto 0) of VECTOR$6 (size1-1 downto 0);
signal tabvect : tableau;
signal number : integer := 0;
begin
	PRO : process (rst, clk, en)

	   begin

	          if rst = '1' then
		              for j in 0 to 9 loop
			      	for i in 0 to (size1-1) loop
			             tabvect(j) (i) <= (others => '0');
				end loop;
			     end loop;
		  elsif (clk'event) and (clk = '1') then
		   if en = '1' then
		       if  number < 10 then
				tabvect(number) <= i;
				number <= number +1;
			else
				number <= 0;
			end if;
		   end if;
		 end if;

end process;

end OPERATEUR;
																  
)

define(`_Gen_entity_sens',`_Gen_entity_sens_not_in_lib($@)')

define(`_Gen_entity_actu',`ifelse($#,6, _Gen_entity_actu_partiel($@),--COMPONENT $1$2 NOT DEFINED; COMPILE IT SEPARATLY
_Gen_entity_actu_not_in_lib($@))
')

define(`_Gen_entity_diffuse',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--****************************  DEFINITION DIFFUSE OPERATION  **************************

entity $1Diffuse is
      generic(       size:integer :=4;
                      tel:integer :=8);
      port (
		rst : std_logic;
		inport : in VECTOR$1 (size-1 downto 0) ;
		outport : out VECTOR$1 (size-1 downto 0)
	);
end $1Diffuse;
	
architecture OPERATEUR of $1Diffuse is
begin
        outport <= inport;

end OPERATEUR;
	
)

define(`_Gen_generic_cin', `ifelse($#, 7,size$1 : integer := $4;
                                tel$1  : integer := $5,
			        size$1 : integer := $4;
			        tel$1  : integer := $5;
			        `_Gen_generic_cin(incr($1),shift(shift(shift(shift(shift(shift(shift($@))))))))')
')
																    
define(`_Gen_port_cin',`ifelse($#, 7,$2 : out VECTOR$6 (size$1 -1 downto 0);
                      rd$3 : out std_logic;
		      ad$3 : in  std_logic, $2 : out VECTOR$6 (size$1 -1 downto 0);
				rd$3 : out std_logic;
				ad$3 : in  std_logic;
				`_Gen_port_cin(incr($1),shift(shift(shift(shift(shift(shift(shift($@))))))))')
')
define(`_Signal_archi_cin', signal sig$1 : VECTOR$5($3-1 downto 0);
		signal flag$6 : std_logic := '1';
		`ifelse($#, 6,,`_Signal_archi_cin(shift(shift(shift(shift(shift(shift($@)))))))')
')

define(`_Signal_archi_cout', signal flag$6 : std_logic :='0';
			`ifelse($#, 6,,`_Signal_archi_cout(shift(shift(shift(shift(shift(shift($@)))))))')
')


define(`_Def_output_cin', $1 <= sig$1;
		rd$2 <= ru and flag$6 ;
		`ifelse($#, 6,,`_Def_output_cin(shift(shift(shift(shift(shift(shift($@)))))))')
')

define(`_Def_output_cout', au$2 <= ad and flag$6;
		`ifelse($#, 6,,`_Def_output_cout(shift(shift(shift(shift(shift(shift($@)))))))')
')

define(`_Def_au_cin', `ifelse($#,6,, or ad$8 `_Def_au_cin(shift(shift(shift(shift(shift(shift($@)))))))')
')

define(`_Def_rd_cout', `ifelse($#,6,, or ru$8 `_Def_rd_cout(shift(shift(shift(shift(shift(shift($@)))))))')
')

dnl define(`_Def_proc_cout', `ifelse($#,6,, `, ru$8 _Def_proc_cout(shift(shift(shift(shift(shift(shift($@)))))))')
dnl')


dnl define(`_Init_cin', for i in 0 to ($3-1) loop
dnl			sig$1(i) <= (others => '0');
dnl		    end loop;
dnl		    `ifelse($#, 6,,`_Init_cin(shift(shift(shift(shift(shift(shift($@)))))))')
dnl')


define(`_Fct_intermediaire', '0' `when $1,')

dnl define(`_Decr_for_flag', `ifelse($1,$2,,`_Fct_intermediaire($1)'
dnl			`_Decr_for_flag(incr($1),$2)')
dnl ')


dnl define(`_Down_flag', `ifelse($#, 6,,`_Fct_intermediaire($12)'
dnl				    `_Down_flag(shift(shift(shift(shift(shift(shift($@)))))))')
dnl ')
dnl define(`_Process_archi_cin', elsif $1(0) = $8 then  sig$3 <= $2; flag$8 <= '1'; flagnotsymetric <= '0'; `_Decr_for_flag(0,$8)'  `_Down_flag(shift(shift($@)))'
dnl		`ifelse($#, 8,else `_Decr_for_flag(0,incr($8))',`_Process_archi_cin($1,$2,shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))')
dnl')

define(`_Select_archi_cin_port', `with conv_integer($1(0)) select
					sig$3 <= $2 when $8,
							sig$3 when others;
		ifelse($#,8,,`_Select_archi_cin_port($1,$2,shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))')
')

define(`_Select_archi_cout_port', `$1 when $6,
				ifelse($#,6,,`_Select_archi_cout_port(shift(shift(shift(shift(shift(shift($@)))))))')
')					

define(`_Select_archi_cin_flag', `with conv_integer($1(0)) select
                                        flag$7 <= ''1'` when $7,
					          ''0'` when others;
		ifelse($#,7,,`_Select_archi_cin_flag($1,shift(shift(shift(shift(shift(shift(shift($@))))))))')
')

						  

define(`_Select_archi_cin_flagns', `ifelse($#,6,,`_Fct_intermediaire($12)'
						`_Select_archi_cin_flagns(shift(shift(shift(shift(shift(shift($@)))))))')
')
							


dnl define(`_Process_archi_cout', elsif ru$3 = '1'  then  sig$1 <= $2; flag$7 <= '1'; `_Decr_for_flag(0,$7)' `_Down_flag(shift($@))'
dnl		`ifelse($#, 7,,`_Process_archi_cout($1,shift(shift(shift(shift(shift(shift(shift($@))))))))')
dnl')

define(`_Gen_entity_cin',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************
LIBRARY WORK;
USE WORK.definitions.all;

--****************************  DEFINITION CONDITION OPERATION IN  **************************

entity $1 is
	GENERIC (
                    size1 : integer := $3;
		    tel1  : integer := $4;
		    size2 : integer := $7;
		    tel2  : integer := $8;
		    `_Gen_generic_cin(3,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
																				                        );
	PORT (
	            rst : in std_logic;
		    $2  : in VECTOR$5 (size1 -1 downto 0);
		    $6  : in VECTOR$9 (size2 -1 downto 0);
		    ru  : in std_logic;
		    au  : out std_logic;
		    `_Gen_port_cin(3,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
  		);
end $1;

architecture OPERATEUR of $1 is
	
	signal flagnotsymetric : std_logic := '0';
	`_Signal_archi_cin(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
	begin
	au <= flagnotsymetric or ad$11 `_Def_au_cin(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))';
	`_Def_output_cin(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
dnl		`_Init_cin(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'  
	`_Select_archi_cin_port($2,$6,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
	`_Select_archi_cin_flag($2,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
	
	`with conv_integer($2(0)) select
			flagnotsymetric <= ''0'` when $15,	
					_Select_archi_cin_flagns(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))
		                	''1'` when others;

	 
	
end OPERATEUR;
')																										
define(`_Gen_port_cout',`ifelse($#, 7,$2 : in VECTOR$6 (size$1 -1 downto 0);
                                 ru$3 : in std_logic;
				 au$3 : out std_logic, $2 : in VECTOR$6 (size$1 -1 downto 0);
				 ru$3 : in std_logic;
				 au$3 : out std_logic;
				 `_Gen_port_cout(incr($1),shift(shift(shift(shift(shift(shift(shift($@))))))))')
')
																									    

define(`_Gen_entity_cout',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
USE IEEE.std_logic_arith.all;
--****************************************************
LIBRARY WORK;
USE WORK.definitions.all;

--****************************  DEFINITION CONDITION OPERATION OUT  **************************

entity $1 is
	GENERIC (
		size1 : integer := $3;
		tel1  : integer := $4;
		size2 : integer := $7;
		tel2  : integer := $8;
		`_Gen_generic_cin(3,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
		);
	PORT (
			rst : in std_logic;
			$2  : in VECTOR$5 (size1 -1 downto 0);
			$6  : out VECTOR$9 (size2 -1 downto 0);
			rd  : out std_logic;
			ad  : in std_logic;
			`_Gen_port_cout(3,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
		);
end $1;

architecture OPERATEUR of $1 is
	signal sig$6 : VECTOR$9($7-1 downto 0);	
	`_Signal_archi_cout(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
	begin
	$6 <= sig$6;
	rd <= ru$11 `_Def_rd_cout(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))';
	`_Def_output_cout(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
	with conv_integer($2(0)) select
			sig$6 <= 
				`_Select_archi_cout_port(shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
				sig$6 when others;
				
	`_Select_archi_cin_flag($2,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))'
end OPERATEUR;

)
																										


define(`_Gen_entity_mul',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--*******************************  DEFINITION MULTIPLICATION OPERATION ******************
entity $1Arit_mul is
  	generic (     	size1 : integer := $5;
                	tel1  : integer := $6;
			size2 : integer := $10;
			tel2  : integer := $11;
			size3 : integer := $15;
			tel3  : integer := $16
		);
	port (          rst : in std_logic;
			b : in VECTOR$7(size1-1 downto 0);
			a : in VECTOR$12(size2-1 downto 0);
			o : out VECTOR$17(size3-1 downto 0)
		);
end $1Arit_mul;

architecture OPERATEUR of $1Arit_mul is
signal sortiemul        : signed(2*tel1-1 downto 0);

begin
  sortiemul <= signed(a(0)) * signed(b(0));
    o(0) <= std_logic_vector(sortiemul(tel3-1 downto 0));

end OPERATEUR;
)

define(`_Gen_entity_add',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--******************************   DEFINITION ADDITION OPERATION   *********************
entity $1Arit_add is
  	generic(      	size1 : integer := $5;
                  	tel1  : integer := $6;
		        size2 : integer := $10;
			tel2  : integer := $11;
			size3 : integer := $15;
			tel3  : integer := $16
		);
	port (
			rst : in std_logic;
			b : in VECTOR$7 (size1 -1 downto 0) ;
			a : in VECTOR$12 (size2 -1 downto 0) ;
			o : out VECTOR$17 (size3 -1 downto 0)
		);
end $1Arit_add;

architecture OPERATEUR of $1Arit_add is
signal sortieadd        : signed(tel3-1 downto 0);
begin
        sortieadd <= signed(a(0)) + signed(b(0));
	o(0) <= std_logic_vector(sortieadd);

end OPERATEUR;
		
)

define(`_Gen_entity_sub',
--***************   DEFINITION OF LIBRARYS  **********

LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
--****************************************************

LIBRARY WORK;
USE WORK.definitions.all;


--******************************  DEFINITION SUBSTRACTION OPERATION  *****************

entity $1Arit_sub is
  	generic(      	size1 : integer := $5;
                  	tel1  : integer := $6;
		        size2 : integer := $10;
			tel2  : integer := $11;
			size3 : integer := $15;
			tel3  : integer := $16
		);
	port (
			rst : in std_logic;
			b : in VECTOR$7 (size1 -1 downto 0) ;
			a : in VECTOR$12 (size2 -1 downto 0) ;
			o : out VECTOR$17 (size3 -1 downto 0)
		);
end $1Arit_sub;

architecture OPERATEUR of $1Arit_sub is
signal sortiesub        : signed(tel3-1 downto 0);
begin
        sortiesub <= signed(b(0)) - signed(a(0));
	        o(0) <= std_logic_vector(sortiesub);

end OPERATEUR;
		
)

define(`_Gen_entity_window',
--******************************  DEFINITION OF LIBRARYS   ******************
LIBRARY IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
USE IEEE.numeric_std.all;
--******************************************************

LIBRARY WORK;
USE WORK.definitions.all;

--************************************  DEFINITION WINDOW OPERATION ******************

entity $1window is
	generic(	size1 : integer := $5;
	                tel1 : integer := $6;
			size2 : integer := $10;
			tel2 : integer := $11);
			
	port (    	clk : in std_logic;
			en  : in std_logic;
			i   : in VECTOR$7(size1-1 downto 0);
			o   : out VECTOR$12(size2-1 downto 0);
			rst : in std_logic);
	 end $1window;

architecture OPERATEUR of floatwindow is
	
signal tabvect : VECTORfloat(size2-1 downto 0) ;
begin
o <= tabvect ;

process(rst,clk,en)
	begin
	 if rst = '1' then
	  for i in 0 to (size2-1) loop
	   tabvect(i) <= (others => '0');
	  end loop;
	 elsif (clk'event) and (clk = '1') then
	 	if en = '1' then
	     	tabvect(size2-2 downto 0) <= tabvect(size2-1 downto 1);
	     	tabvect(size2-1) <= i(0);
	 	end if;
	end if;
end process;


end OPERATEUR;
)	  
	   

define(`_Gen_entity_opn',
`
ifelse($2,Arit_mul,`_Gen_entity_mul($@)',
	ifelse($2,Arit_add,`_Gen_entity_add($@)',
		ifelse($2,Arit_sub,`_Gen_entity_sub($@)',
			ifelse($2,window,`_Gen_entity_window($@)',`_Gen_entity_opn_not_in_lib($@)'))))

')

define(`_Gen_entity',
--***************   DEFINITION OF ENTITY   **********************
ENTITY $1 IS
	PORT (
		clk : in std_logic;
		rst : in std_logic
	     );

END $1;
)

define(`_Gen_comp_uci', 
`--**************   COMPONENT DEFINITIONS   *********************
--************** COMPONENT uc_infini *********************
ARCHITECTURE $1 OF $2 is
	COMPONENT uc_infini 
		PORT (
			rst : in std_logic;
			en  : out std_logic;
			rsu : in std_logic;
			asd : in std_logic;
			rfu : in std_logic;
			afu : out std_logic;
			rfd : out std_logic;
			afd : in std_logic
		);
		
	END COMPONENT;

')

define(`_Gen_comp_uc',
`--**************   COMPONENT uc   *********************
	COMPONENT uc 
		GENERIC (	nb_bit_compteur  : integer := 3);

		  PORT (	reset  : in std_logic;
		  	 	en   : out std_logic;
				clk  : in std_logic;
		        	rsu  : in std_logic;
				asu  : out std_logic;
				rsd  : out std_logic;
		        	asd  : in std_logic;
		        	rfu  : in std_logic;
		        	afu  : out std_logic;
				rfd  : out std_logic;
		        	afd  : in std_logic;
				cpt  : out std_logic_vector(nb_bit_compteur-1 downto 0)
				);
	END COMPONENT;
')


define(`_Gen_comp_xseq',
`--***************** COMPONENT Xseq    ********************
	COMPONENT $1Xseq 
		GENERIC (	insize  : integer := 3;
		  		outsize : integer := 1;
		  		nel : integer := 1;
	  	  		tel : integer := 8
			);
		PORT ( 		
		  		rst : in std_logic;
				inport  : in VECTOR$1 (insize-1 downto 0) ;
				outport : out VECTOR$1 (outsize-1 downto 0);
	    			cpt : in std_logic_vector(nel-1 downto 0)
		);
	END COMPONENT;

')

define(`_Gen_comp_mseq',
`--***************  COMPONENT Mseq   ***********************
	COMPONENT $1Mseq 
		GENERIC (	insize  : integer := 1;
		  		outsize : integer := 3;
		  		nel : integer := 1;
	  	  		tel : integer := 8
			);      
	      	PORT (	
		  		rst    : in std_logic;
				inport : in VECTOR$1 (insize-1 downto 0) ;
				clk : in std_logic;
				outport : out VECTOR$1 (outsize-1 downto 0) ;
	    			cpt : in std_logic_vector(nel-1 downto 0);
				en  : in std_logic
			);
	END COMPONENT;
')

define(`_Gen_comp_iterate',
`--************************  COMPONENT ITERATE ********************
	COMPONENT  $1Iterate  
		GENERIC (	size : integer :=1;
  				nel  : integer :=3;
				tel  : integer :=8
				);
		PORT (	
		  		rst    : in std_logic;
				I_init : in VECTOR$1 (size-1 downto 0) ;
				inport : in VECTOR$1 (size-1 downto 0) ;
				clk    : in std_logic;
				O_si   : out VECTOR$1 (size-1 downto 0) ;
				outport : out VECTOR$1 (size-1 downto 0) ;
	    			cpt    : in std_logic_vector(nel-1 downto 0);
				en     : in std_logic
		);
	END COMPONENT;
')

define(`_Gen_comp_diffuse',
`--**********************   COMPONENT DIFFUSE ****************
	COMPONENT $1Diffuse 
		GENERIC (	size : integer :=1;
				tel  : integer :=8
			);
		PORT (
				rst : in std_logic;
				inport  : in VECTOR$1 (size-1 downto 0) ;
				outport : out VECTOR$1 (size-1 downto 0) 
		);
	END COMPONENT;

')


define(`_Gen_comp_cin',
`--*****************************  COMPONENT CONDITION IN *************
	COMPONENT $1
		GENERIC (
				size1 : integer := $3;
				tel1  : integer := $4;
				size2 : integer := $7;
				tel2  : integer := $8;
				_Gen_generic_cin(3,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))
			);
		PORT (
				rst : in std_logic;
				$2  : in VECTOR$5 (size1 -1 downto 0);
				$6  : in VECTOR$9 (size2 -1 downto 0);
				ru  : in std_logic;
				au  : out std_logic;
				_Gen_port_cin(3,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))
			);
	END COMPONENT;
')

define(`_Gen_port_cout',`ifelse($#, 7,$2 : in VECTOR$6 (size$1 -1 downto 0);
				      ru$3 : in std_logic;
				      au$3 : out std_logic, $2 : in VECTOR$6 (size$1 -1 downto 0);
				      			    ru$3 : in std_logic;
							    au$3 : out std_logic;
	`_Gen_port_cout(incr($1),shift(shift(shift(shift(shift(shift(shift($@))))))))')
')

define(`_Gen_comp_cout',
`--*****************************  COMPONENT CONDITION OUT *************
	COMPONENT $1
		GENERIC (
				size1 : integer := $3;
				tel1  : integer := $4;
				size2 : integer := $7;
				tel2  : integer := $8;
				_Gen_generic_cin(3,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))
			);
		PORT (
				rst : in std_logic;
				$2  : in VECTOR$5 (size1 -1 downto 0);
				$6  : out VECTOR$9 (size2 -1 downto 0);
				rd  : out std_logic;
				ad  : in std_logic;
				_Gen_port_cout(3,shift(shift(shift(shift(shift(shift(shift(shift(shift($@))))))))))
			);
	END COMPONENT;
')
				
define(`_Gen_generic', `ifelse($#, 5, size$1 : integer := $3;
			        tel$1  : integer := $4,  
				    size$1 : integer := $3;
				tel$1  : integer := $4;
				`_Gen_generic(incr($1),shift(shift(shift(shift(shift($@))))))')
')


define(`_Gen_port_s', `ifelse($#, 5, $2 : out VECTOR$5 (size$1 -1 downto 0) , $2 : out VECTOR$5 (size$1 -1 downto 0) ;
	 `_Gen_port_s(incr($1),shift(shift(shift(shift(shift($@))))))')
')

define(`_Gen_comp_sens', 
`--************** COMPONENT SENSOR  *********************
	COMPONENT $1$2 
		GENERIC (	
				_Gen_generic(1,shift(shift($@)))
			);
		PORT (
				rst : in std_logic;
				clk    : in std_logic;
				en     : in std_logic;
				_Gen_port_s(1,shift(shift($@)))
		     );
	END COMPONENT;
')

define(`_Gen_comp_const',
`--************** COMPONENT CONSTANTE  *********************
	COMPONENT $1$2 
		GENERIC (
				_Gen_generic(1,shift(shift($@)))
			);
		PORT (
			_Gen_port_s(1,shift(shift($@)))
		     );
	END COMPONENT;
')

define(`_Gen_port_a', `$2 : in VECTOR$5 (size$1 -1 downto 0) ;
				ifelse($#, 5,,`_Gen_port_a(incr($1),shift(shift(shift(shift(shift($@))))))')
')
define(`_Gen_comp_actu',
`--*************************   COMPONENT ACTUATOR    *****************
	COMPONENT $1$2 
		GENERIC (
				_Gen_generic(1,shift(shift($@)))
			);
		PORT (
				rst : in std_logic;
				clk : in std_logic;
				_Gen_port_a(1,shift(shift($@)))
				en  : in std_logic
		     );
	END COMPONENT;
')

define(`_Gen_generic_opn', `ifelse($#, 6,size$1 : integer := $4;
			        tel$1  : integer := $5,   
				    size$1 : integer := $4;
			        tel$1  : integer := $5;
				`_Gen_generic_opn(incr($1),shift(shift(shift(shift(shift(shift($@)))))))')
')
define(`_Gen_port_opn', `$2 : $3 VECTOR$6 (size$1 -1 downto 0) ;
				ifelse($#, 6,,`_Gen_port_opn(incr($1),shift(shift(shift(shift(shift(shift($@)))))))')
')


define(`_Gen_comp_opn', 
`ifelse($2,window,
--**************************  COMPONENT WINDOW ***********************
	COMPONENT $1$2
		GENERIC (
				_Gen_generic_opn(1, shift(shift($@)))
			);
		PORT (
				_Gen_port_opn(1,shift(shift($@)))
				rst : in std_logic;
				clk : in std_logic;
				en  : in std_logic
		      );
	END COMPONENT;,
--************************    COMPONENT OPERATOR   **********************
	COMPONENT $1$2 
		GENERIC (
				_Gen_generic_opn(1, shift(shift($@)))
			);
		PORT (
				_Gen_port_opn(1,shift(shift($@)))
				rst : in std_logic
		     );
	END COMPONENT;)
')
define(`_Gen_comp_memory',
`--******************************* COMPONENT MEMORY  *********************
	COMPONENT $1$2
		GENERIC (
				_Gen_generic_opn(1, shift(shift($@)))
			);
		PORT (
				_Gen_port_opn(1,shift(shift($@)))
				rst : in std_logic;
				clk : in std_logic;
				en : in std_logic
			);
		END COMPONENT;
')
				
define(`_Sig_array',`signal $1 : VECTOR$4 ( $2 -1 downto 0); ')
define(`_Sig_std', `signal $1 : std_logic;')
define(`_Sig_vect',`signal $1 : std_logic_vector ($2 -1 downto 0);')


define(`_Cnx_uc_inf_combi',
signal highstate : std_logic :='1';
signal lowstate : std_logic := '0';
`--************************  CONNEXION UC INFINITE FOR APPLICATION ONLY COMBINATOIRE  ***************
 -- Liste des operations controlees par $1 uc  : $2
 -- Liste des uc situees du cote fast en amont de $1 uc : $3
 -- Liste des uc situees du cote fast en aval de $1 uc  : $4
 begin
 	$1te : uc_infini
		PORT MAP (
			rst => rst,
			en  => enableinf,
			rsu => highstate,
			asd => highstate,
			rfu => lowstate,
			afu => afuinfini,
			rfd => rfdinfini,
			afd => highstate
			);
')
			
define(`_Cnx_uc_inf',
signal highstate : std_logic :='1';
`--************************  CONNEXION UC INFINITE FOR APPLICATION WITH SEQUENTIEL OPERATORS  ***************
 -- Liste des operations controlees par $1 uc  : $2
 -- Liste des uc situees du cote fast en amont de $1 uc : $3
 -- Liste des uc situees du cote fast en aval de $1 uc  : $4
 begin 
 	$1te : uc_infini
		PORT MAP (
			rst => rst,
			en  => enableinf,
			rsu => highstate,
			asd => highstate,
			rfu => $5,
			afu => afuinfini,
			rfd => rfdinfini,
			afd => $7
			);
	ifelse($5,$6,, $5 <= $6;)
	ifelse($7,$8,, $7 <= $8;)
')

define(`_Cnx_uc',
`--************************  CONNEXION UC FINITE FOR APPLICATION WITH SEQUENTIEL OPERATORS  ***************
 -- Liste des operations controlees par $1 uc  : $2
 -- Liste des uc situees du cote fast en amont de $1 uc : $3
 -- Liste des uc situees du cote fast en aval de $1 uc  : $4

 	$1 : uc
		GENERIC MAP ($5)
		PORT MAP (
			reset => rst,
			en  => enable$6,
			clk => clk,
			rsu => $7,
			asu => asu$6,
			rsd => rsd$6,
			asd => $9,
			rfu => $11,
			afu => afu$6,
			rfd => rfd$6,
			afd => $13,
			cpt => compteur$6
			);
		ifelse($7,$8,, $7 <= $8;)
		ifelse($9,$10,, $9 <= $10;)
		ifelse($11,$12,, $11<= $12;)
		ifelse($13,$14,, $13<= $14;)
')


define(`_Cnx_xseq', 
`--**************************** CONNEXION OF Xseq OPERATOR ********************
 -- La reference de loperation Xseq : $2
 -- Le chemin de loepation Xseq : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4
 
 	$2 : $1Xseq
		GENERIC MAP ($5, $6, $7, $8)
		PORT MAP (
			rst     => rst,
			inport  => $9,
			outport => $10,
			cpt     => compteur$4
			);
')

define(`_Cnx_mseq',
`--**************************** CONNEXION OF Mseq OPERATOR ********************
 -- La reference de loperation Mseq : $2
 -- Le chemin de loepation Mseq : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4

 	$2 : $1Mseq
		GENERIC MAP ($5, $6, $7, $8)
		PORT MAP (
			rst     => rst,
			inport  => $9,
			outport => $10,
			cpt     => compteur$4,
			clk     => clk,
			en      => enable$4
			);
')

define(`_Cnx_iterate',
`--**************************** CONNEXION OF Iterate OPERATOR ********************
 -- La reference de loperation Iterate : $2
 -- Le chemin de loepation Iterate : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4
 
 	$2 : $1Iterate
		GENERIC MAP ($5, $6, $7)
		PORT MAP (
			rst     => rst,
			clk     => clk,
			en      => enable$4,
			cpt     => compteur$4,
			I_init  => $8,
			inport  => $9,
			O_si    => $10,
			outport => $11
			);
')


define(`_Cnx_diffuse',
`--**************************** CONNEXION OF Diffuse OPERATOR ********************
 -- La reference de loperation Diffuse : $2
 -- Le chemin de loepation Diffuse : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4

 	$2 : $1Diffuse
		GENERIC MAP ($5, $6)
		PORT MAP (
			rst    => rst,
			inport  => $7,
			outport => $8
			);
')



define(`_Cnx_generic', `ifelse($#,4,`$3, $4 )', `$3, $4, _Cnx_generic(shift(shift(shift(shift($@)))))')
')

define(`_Cnx_port_s_a', `ifelse($#,4,$1 => $2, `$1 => $2,
		_Cnx_port_s_a(shift(shift(shift(shift($@)))))')
')

define(`_Cnx_sens',
`--**************************** CONNEXION OF SENSOR OPERATOR ********************
 -- La reference de loperation Sensor : $2
 -- Le chemin de loperation Sensor : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4

 	$2 : $1$6
		GENERIC MAP ( _Cnx_generic(shift(shift(shift(shift(shift(shift($@))))))) 
		PORT MAP (
			rst => rst,
			clk => clk,
			en  => $5,
			_Cnx_port_s_a(shift(shift(shift(shift(shift(shift($@)))))))
			);
')

define(`_Cnx_cst',
`--**************************** CONNEXION OF CONSTANTE OPERATOR ********************
 -- La reference de loperation Constante : $2
 -- Le chemin de loperation Constante : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4

 	$2 : $1$6
		GENERIC MAP ( _Cnx_generic(shift(shift(shift(shift(shift(shift($@)))))))
		PORT MAP (
			_Cnx_port_s_a(shift(shift(shift(shift(shift(shift($@)))))))
			);
')


define(`_Cnx_actu',
`--**************************** CONNEXION OF ACTUATOR OPERATOR ********************
 -- La reference de loperation Actuator : $2
 -- Le chemin de loperation Actuator : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4
 
	$2 : $1$6
		GENERIC MAP ( _Cnx_generic(shift(shift(shift(shift(shift(shift($@)))))))
		PORT MAP (
			rst => rst,
			clk => clk,
			en  => $5,
			_Cnx_port_s_a(shift(shift(shift(shift(shift(shift($@)))))))
			);
')

define(`_Cnx_port_cout_combi', `ifelse($#,6,`$2 => $3,
					    au$6 => au$6$1,
					    ru$6 => highstate', `$2 => $3,
					    			au$6 => au$6$1,
								ru$6 => highstate,
				_Cnx_port_cout_combi($1,shift(shift(shift(shift(shift(shift($@)))))))')
')

define(`_Cnx_cout_combi',
`--*****************************  CONNEXION OF CONDITION OUT OPERATOR FOR APPLICATION PUREMENT COMBINATOIRE **************
 -- La reference de loperation Cout : $2
 -- Le chemin de loperation Cout : $3
 -- Cette operation appartient a la frontiere numero $4 

 	$2_1 : $2
		GENERIC MAP ($7,$8,$11,$12,_Cnx_generic_cin_combi(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))
		PORT MAP (
			rst => rst,
			$5  => $6,
			$9  => $10,
			ad  => highstate,
			rd  => rd$2,
			_Cnx_port_cout_combi($2,shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))
			);
')

define(`_Cnx_generic_cin_combi', `ifelse($#,5,`$3, $4 )', `$3, $4, _Cnx_generic_cin_combi(shift(shift(shift(shift(shift($@))))))')
')

define(`_Cnx_port_cin_combi', `ifelse($#,6,`$2 => $3,
					    rd$6 => rd$6$1,
					    ad$6 => highstate', `$2 => $3,
					    			 rd$6 => rd$6$1,
								 ad$6 => highstate,
				_Cnx_port_cin_combi($1,shift(shift(shift(shift(shift(shift($@)))))))')
')

define(`_Cnx_cin_combi',
`--*****************************  CONNEXION OF CONDITION IN OPERATOR FOR APPLICATION PUREMENT COMBINATOIRE **************
 -- La reference de loperation Cin : $2
 -- Le chemin de loperation Cin : $3
 -- Cette operation appartient a la frontiere numero $4 

 	$2_1 : $2
		GENERIC MAP ($7,$8,$11,$12,_Cnx_generic_cin_combi(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))
		PORT MAP (
			rst => rst,
			$5  => $6,
			$9  => $10,
			ru  => highstate,
			au  => au$2,
			_Cnx_port_cin_combi($2,shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))
			);
')
		
			

define(`_Cnx_generic_cin', `ifelse($#,7,`$3, $4 )', `$3, $4, _Cnx_generic_cin(shift(shift(shift(shift(shift(shift(shift($@))))))))')
')

define(`_Cnx_port_cin', `ifelse($#,8,`$2 => $3,
			rd$6 => rd$6$1,
			ad$6 => $7', `$2 => $3,
			rd$6 => rd$6$1,
			ad$6 => $7,
			_Cnx_port_cin($1,shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))')
')

define(`_Cnx_def_signal',`ifelse($#,7,`ifelse($6,$7,, $6 <= $7;)',`ifelse($6,$7,, $6 <= $7;)
			_Cnx_def_signal(shift(shift(shift(shift(shift(shift(shift($@))))))))')
')

define(`_Cnx_cin',
`--*****************************  CONNEXION OF CONDITION IN OPERATOR  **************
 -- La reference de loperation Cin : $2
 -- Le chemin de loperation Cin : $3
 -- Cette operation appartient a la frontiere numero $4 

 	$2_1 : $2
		GENERIC MAP ($7,$8,$11,$12,_Cnx_generic_cin(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))))
		PORT MAP (
			rst => rst,
			$5  => $6,
			$9  => $10,
			ru  => $13,
			au  => au$2,
			_Cnx_port_cin($2,shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))))
			);
		ifelse($13,$14,, $13 <= $14;)
		_Cnx_def_signal(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))))
		
')

define(`_Cnx_port_cout', `ifelse($#,8,`$2 => $3,
			au$6 => au$6$1,
			ru$6 => $7', `$2 => $3,
			au$6 => au$6$1,
			ru$6 => $7,
			_Cnx_port_cout($1,shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))')
')

define(`_Cnx_cout',
`--*****************************  CONNEXION OF CONDITION OUT OPERATOR  **************
 -- La reference de loperation Cout : $2
 -- Le chemin de loperation Cout : $3
 -- Cette operation appartient a la frontiere numero $4 

 	$2_1 : $2
		GENERIC MAP ($7,$8,$11,$12,_Cnx_generic_cin(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))))
		PORT MAP (
			rst => rst,
			$5  => $6,
			$9  => $10,
			rd  => rd$2,
			ad  => $13,
			_Cnx_port_cout($2,shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))))
			);
		ifelse($13,$14,, $13 <= $14;)
		_Cnx_def_signal(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift(shift($@)))))))))))))))

')

define(`_Cnx_opn',
`ifelse($5,window,
	ifelse($4,infini,
`--**************************** CONNEXION OF WINDOW   *********************
 -- La reference de loperation : $2
 -- Le chemin de loperation : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4
 
 	$2 : $1$5
	GENERIC MAP ( _Cnx_generic(shift(shift(shift(shift(shift($@))))))
	PORT MAP (
		rst => rst,
		en  => enableinf,
		clk => clk,
		_Cnx_port_s_a(shift(shift(shift(shift(shift($@))))))
		);',
`--**************************** CONNEXION OF WINDOW   *********************
 -- La reference de loperation : $2
 -- Le chemin de loperation : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4

 	$2 : $1$5
	GENERIC MAP ( _Cnx_generic(shift(shift(shift(shift(shift($@))))))
	PORT MAP (
		rst => rst,
		clk => clk,
		en  => enable$4,
		_Cnx_port_s_a(shift(shift(shift(shift(shift($@))))))
		);
		'),
`--**************************** CONNEXION OF OPERATOR ********************
 -- La reference de loperation : $2
 -- Le chemin de loperation : $3
 -- Cette operation appartient a la frontiere numero $4 donc de uc $4

	$2 : $1$5
	GENERIC MAP ( _Cnx_generic(shift(shift(shift(shift(shift($@))))))
	PORT MAP (
		rst => rst,
		_Cnx_port_s_a(shift(shift(shift(shift(shift($@))))))
		);')
')
define(`_Cnx_memory',
`--**************************  CONNEXION OF MEMORY  ******************************
-- La reference de loperation : $2
-- Le chemin de loperation : $3
-- Cette operation appartient a la frontiere numero $4 donc de uc $4

	$2 : $1$6
	GENERIC MAP ( _Cnx_generic(shift(shift(shift(shift(shift(shift($@)))))))
	PORT MAP (
		rst => rst,
		en  => $5,
		clk => clk,
		_Cnx_port_s_a(shift(shift(shift(shift(shift(shift($@)))))))
		);
')


define(`_End',`END $1;
-- BEGIN COMPILE SCRIPT
--    vcom definition_$1.vhdl
      undivert(5)
--    vcom $1_toplevel.vhdl
')


